/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2021  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#include <ctype.h>
#include <genvector/gds_char.h>

#include "action.h"
#include "script.h"
#include "fgwirc.h"
#include "config.h"

#define FLOOD_INC 8
#define FLOOD_MAX 40

static fgw_obj_t *ctcpobj;
static int ctcp_flood;

static fgw_error_t on_ctcp_tick(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	if (ctcp_flood > 0)
		ctcp_flood--;
	return 0;
}

static fgw_error_t on_ctcp_req(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	char tmp[1800];
	sockev_t sock;
	action_details_t *actd;
	GET_ACTD(actd);

	sock = actd->server->sock;
	if ((strncmp(actd->argv[2], "ACTION", 6) == 0) && (isspace(actd->argv[2][6]))) {
		sprintf(tmp, " * %s %s", actd->from->nick, actd->argv[2]+7);
		irc_win_deliver_incoming(actd->server, actd->from, actd->argv[1], tmp);
		actd->res = ACTION_HOOK_BLOCK;
	}
	else if (strcmp(actd->argv[2], "VERSION") == 0) {
		sprintf(tmp, "[CTCP] version request from %s (%s@%s)", actd->from->nick, actd->from->ident, actd->from->host);
		win_append(actd->server->status_win, tmp);
		if (ctcp_flood < FLOOD_MAX) {
			sockev_printf(sock, "NOTICE %s :\001VERSION fgwirc %s %s\001\r\n", actd->from->nick, FGWIRC_VERSION_STR, FGWIRC_URL_STR);
			ctcp_flood += FLOOD_INC;
		}
		actd->res = ACTION_HOOK_BLOCK;
	}
	else if ((strcmp(actd->argv[2], "PING") == 0) || (strcmp(actd->argv[2], "ECHO") == 0)) {
		char *payload = actd->argv[3];
		if (argc <= 3)
			payload = "";
		sprintf(tmp, "[CTCP] %s request from %s (%s@%s): '%s'", actd->argv[2], actd->from->nick, actd->from->ident, actd->from->host, payload);
		win_append(actd->server->status_win, tmp);
		if (ctcp_flood < FLOOD_MAX) {
			sockev_printf(sock, "NOTICE %s :\001%s %s\001\r\n", actd->from->nick, actd->argv[2], payload);
			ctcp_flood += FLOOD_INC;
		}
		actd->res = ACTION_HOOK_BLOCK;
	}
	return 0;
}

static fgw_error_t act_ctcp(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	win_t *curr = idx2win(win_curr_idx);
	char *target = NULL, *cmd = NULL;
	sockev_t sock;

	ACT_CONVARG(1, FGW_STR, target = argv[1].val.str);
	ACT_CONVARG(2, FGW_STR, cmd = argv[2].val.str);

	sock = irc_server_sock(curr->target.sid);
	if (sock == SOCKEV_INVALID) {
		ACT_ERROR(cmd, "Server not connected\n");
		return FGW_ERR_ARG_CONV;
	}

	if (strcmp(cmd, "version") == 0) cmd = "VERSION";
	if (strcmp(cmd, "ping") == 0) cmd = "PING";

	sockev_printf(sock, "PRIVMSG %s :\001%s\001\r\n", target, cmd);

	return 0;
}


static fgw_error_t act_me(fgw_arg_t *res, int argc, fgw_arg_t *argv)
{
	win_t *win = idx2win(win_curr_idx);
	sockev_t sock = irc_server_sock(win->target.sid);
	gds_t tmp = {0};
	int pure;

	if (sock == SOCKEV_INVALID) {
		win_append(win, "Can not message: server not connected");
		return 0;
	}


	gds_append_str(&tmp, " * ");
	gds_append_str(&tmp, irc_get_win_my_nick(win, irc_nick));
	gds_append(&tmp, ' ');
	pure = tmp.used;
	ACT_GDS_REST(&tmp, 1);

	win_append(win, tmp.array);

	sockev_printf(sock, "PRIVMSG %s :\001ACTION %s\001\r\n", win->target.name, tmp.array+pure);

	gds_uninit(&tmp);
	return 0;
}

void ctcp_init(void)
{
	ctcpobj = fgw_obj_reg(&fgw, "ctcp");

	fgw_func_reg(ctcpobj, "on_ctcp_req", on_ctcp_req);
	fgw_func_reg(ctcpobj, "on_tick_second", on_ctcp_tick);
	fgw_func_reg(ctcpobj, "ctcp", act_ctcp);
	fgw_func_reg(ctcpobj, "me", act_me);
}
