#ifndef WINDOW_H
#define WINDOW_H
#include <stdio.h>
#include <time.h>
#include <tcurses/tcurses.h>
#include <genlist/gendlist.h>
#include <genvector/vtp0.h>
#include "target.h"
#include "global_typedefs.h"

extern tcurses_term_t *term;
extern int win_max_scroll_back_len; /* number of raw lines to keep per window */
extern int win_display_timestamp;
extern vtp0_t windows;
extern int win_curr_idx;

struct wline_s {
	gdl_elem_t link;
	time_t time;
	wline_t *raw;      /* wrapped: points to the raw line it is wrapped for */
	unsigned first:1;  /* wrapped: 1 if it is the first wrapped line of a raw */
	int len;
	char s[1];
};

struct win_s {
	gdl_list_t raw, wrapped; /* of wline_t * entries */
	int wrap_sx;
	wline_t *scroll_start; /* (scroll_start == NULL) exactly when (scroll_pending == 0) */
	int scroll_pending;
	FILE *logf;
	int idx; /* cache for faster lookup */
	unsigned dirty:1; /* got lines appended since last draw */
	unsigned needs_log_flush:1;

	const target_t target; /* affinity: channel to join or nick to query with */

	/* handler - what is displayed in the window */
	char *name; /* dislay name */
};


void win_init(void);
void win_uninit(void);

void win_draw(win_t *win);

void win_scroll_up(win_t *win, int lines);
void win_scroll_down(win_t *win, int lines);

void win_del_oldest(win_t *win);

void win_append_(win_t *win, const char *line);

/* Append a single line built by appending a NULL-terminated list of strings */
void win_append_strs(win_t *win, ...);

const char *win_log(win_t *win, const char *name, int on);
void win_draw(win_t *win);


win_t *win_new(const char *name);
void win_show_idx(int idx);
void win_show(win_t *win);

win_t *idx2win(int idx);

void win_append(win_t *win, const char *line);

extern const char win_target_keep[];
void win_set_target(win_t *win, target_type_t type, int sid, const char *name, const char *key);
void win_set_target_sid(win_t *win, int sid);
void win_set_target_state(win_t *win, target_state_t st);

extern httw_t target2win;

win_t *win_target2win(target_type_t type, int sid, const char *name);

void win_draw_status(win_t *win);
void win_flush(void);

void win_resize(int sx, int sy);
void win_get_screen_size(int *sx, int *sy);

#define no_change_window0() \
	if (win_curr_idx == 0) { \
		ACT_ERROR("server", "can't change window 0 target"); \
		return FGW_ERR_ARG_CONV; \
	}

/* replaces win->name with fgw_strdup(new_name) */
#define win_set_name(win, new_name) do { free(win->name); win->name = fgw_strdup(new_name); win->dirty = 1; } while(0);


#endif
